/*******************************************************************************
 * Copyright 2002 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the Software), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
using System;
using System.Collections;
using System.Reflection;

//##############################################################################
//# CTTCreateUser
class CTTCreateUser
{
	SLoginUser loginUser; // The user to log in as
	SUser newUser;		  // The user/customer to create
	long cookie;		  // Cookie returned after logging into a database
	ttsoapcgi soapServer; // The TestTrack Pro SOAP Server
	private enum eUserType{ user, customer };//Type of user to create

	[STAThread]
	/***************************************************************************
	 * Main
	 */
	static void Main(string[] args)
	{
		if ( (args.Length == 0)
			|| ( (args.Length == 1) && (args[0] == "/h" || args[0] == "-h") ) )
		{
			ShowUsage();
			return;
		}
		CCommandLineWrapper arguments = new CCommandLineWrapper();
		arguments.AddSingleArgs("-c");
		arguments.AddDoubleArgs("-lu","-lp","-d","-f","-l","-un","-ug","-p","-ul");
		if ( arguments.ScanCommandLine( ref args ) )
		{
			CTTCreateUser createUser = new CTTCreateUser();
			if ( createUser.ExtractDataFromArguments( arguments ) )
				createUser.CreateUser();
			else
				ShowUsage();
		}
		else
			ShowUsage();
	}

	/***************************************************************************
	 * CreateUser - Handles the details of creating a user/customer
	 */
	private void CreateUser()
	{
      if ( ! VerifyDatabase() )
      {
         Console.WriteLine("The specified user: {0} is not able to login to database: {1}",
            loginUser.userName, loginUser.database);
         return;
      }
		try 
		{
			cookie = soapServer.DatabaseLogon( loginUser.database, loginUser.userName, loginUser.password );
			if ( CanWorkWithUser( newUser.isCustomer ? eUserType.customer : eUserType.user ) )
			{
            if( ! newUser.isCustomer && newUser.userName == null )
            {
               Console.WriteLine("Users must have a Username and Group.");
               return;
            }

				CUser user = new CUser();

				// Note: The following three options must be set to true if you want the
            // user/customer to be able to login to TestTrack Pro
            user.isglobal = ! (newUser.userName == null);
            user.isactiveglobal = ! (newUser.userName == null);
            user.isactive = true;
				user.firstname = newUser.firstName;
				user.lastname = newUser.lastName;
				user.loginname = newUser.userName;
				user.password = newUser.password;
				user.usergroupname = newUser.userGroup;
            user.userLicense = newUser.userLicense;
				if ( user.iscustomer = newUser.isCustomer )
					soapServer.addCustomer(cookie, user);
				else
					soapServer.addUser(cookie, user);
			}
			else
			{
				Console.WriteLine("You do not have permission to add {0}.",
					newUser.isCustomer ? "customers" : "users");
			}
		}
		catch(System.Exception e)
		{
			Console.WriteLine( e.Message );
		}
		finally
		{
			// Disconnect
			if ( cookie != 0 )
			{
				try 
				{
					soapServer.DatabaseLogoff( cookie );
				} 
				catch (System.Exception e) 
				{
					Console.WriteLine( "Error while attempting to disconnect, error message is: {0}", e.Message );
				}
			}
		}
	}

	/***************************************************************************
	 * ShowUsage - Shows program usage
	 */
	static void ShowUsage()
	{
		Module module = Assembly.GetExecutingAssembly().GetModules()[0];
		string progName = module.Name.Substring(0, module.Name.Length - 4).ToUpper();
		Console.WriteLine("{0}\r\n" +
						"   -lu username    User to log in as\r\n" +
						"  [-lp password]   Password of the above user, if blank omit\r\n" +
						"   -d database     Database to log into\r\n" +
						"   -c              Create user as customer\r\n" +
						"   -f first name   First name of created user\r\n" +
						"   -l last name    Last name of created user\r\n" +
						"  [-un user name]  User name for created user\r\n" +
						"  [-ug user group] User group for created user\r\n" +
						"  [-p password]    Password for new user\r\n" +
                  "  [-ul license]    License type for new user (Floating, Named)"
			, progName);
	}

	/***************************************************************************
	 * ExtractDataFromArguments
	 */
	public bool ExtractDataFromArguments( CCommandLineWrapper args )
	{
		bool result = true;
		Hashtable lookup = args.GetHashTable();

		// Verify parameters
		if ( ! lookup.ContainsKey("-lu") )
		{
			Console.WriteLine("Error, login user not specified,");
			result = false;
		}
		if ( ! lookup.ContainsKey("-d") )
		{
			Console.WriteLine("Error, database not specified.");
			result = false;
		}
		if ( (lookup.ContainsKey("-f") == false) && (lookup.ContainsKey("-l") == false ) )
		{
			Console.WriteLine("Error, first and/or last name not specified.");
		}

		// Failed validation?
		if ( ! result )
			return result;

		/*** Get data ***/
		// Login user parameters
		loginUser.userName = (string)lookup["-lu"];
		loginUser.password = (string)lookup["-lp"];
		loginUser.database = (string)lookup["-d"];
		// Created user parameters
		newUser.isCustomer = lookup.ContainsKey("-c");
		newUser.firstName = (string)lookup["-f"];
		newUser.lastName = (string)lookup["-l"];
		newUser.userGroup = (string)lookup["-ug"];
		newUser.userName = (string)lookup["-un"];
		newUser.password = (string)lookup["-p"];
      newUser.userLicense = (string)lookup["-ul"];

		return result;
	}

	/***************************************************************************
	 * CTTCreateUser (constructor)
	 */
	CTTCreateUser()
	{
		loginUser = new SLoginUser();
		newUser = new SUser();
		soapServer = new ttsoapcgi();
	}

	/***************************************************************************
	 * VerifyDatabase - Verifies if the database specified exists on the server
	 */
	private bool VerifyDatabase()
	{
		bool result = false;
		CDatabase[] dbList = soapServer.getDatabaseList();
		foreach (CDatabase db in dbList )
		{
			if ( db.name == loginUser.database )
			{
				result = true;
				break;
			}
		}
		return result;
	}

	/***************************************************************************
	 * CanWorkWithUser - Determines if you can edit users or customers based on
	 *                   the type passed in
	 */
	private bool CanWorkWithUser( eUserType userType )
	{
		bool result = false;
		CDatabaseTable[] tableList = soapServer.getTableList(cookie);
		foreach ( CDatabaseTable table in tableList )
		{
			if ( table.name == ( userType == eUserType.user ? "User":"Customer" ) )
			{
				result = true;
				break;
			}
		}
		return result;
	}
}