/*******************************************************************************
 * Copyright 2002 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the Software), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
using System;
using System.Collections;
//##############################################################################
//# SLoginUser - Structure used to hold data related to the user to use to log
//#              into the database
struct SLoginUser 
{
	public string userName;
	public string password;
	public string database;
}


//##############################################################################
//# SUser - Structure used to create a new user or customer
struct SUser
{
	public bool isCustomer;
	public string firstName;
	public string lastName;
	public string userName;
	public string password;
	public string userGroup;
   public string userLicense;
}


//##############################################################################
//# CCommandLineWrapper - Utility class used to wrap the command line arguments
class CCommandLineWrapper
{
	/******************************************************************************
	 * AddSingleArgs - Add arguments that serve as flags or sentinels
	 */
	public void AddSingleArgs( params string[] args )
	{
		if ( args != null )
		{
			foreach ( string key in args )
				if ( key != null )
					singleArgs.Add( key );
		}
	}
	
	/******************************************************************************
	 * AddDoubleArgs - Adds arguments that have values, ie it's not a 'flag' type
	 *                 of argument
	 */
	public void AddDoubleArgs( params string[] args )
	{
		if ( args != null )
		{
			foreach ( string key in args )
			{
				if ( key != null )
					doubleArgs.Add( key );
			}
		}
	}

	/******************************************************************************
	 * ScanCommandLine - Scans the given command line for arguments previously
	 *                   added via AddSingleArgs and AddDoubleArgs
	 */
	public bool ScanCommandLine( ref string[] args )
	{
		bool result = true;
		if ( args != null )
		{
			for ( int i = 0 ; i < args.Length && result ; i++ )
			{
				if ( singleArgs.IndexOf( args[i] ) != -1 )
					lookupTable.Add( args[i], null );
				else if ( doubleArgs.IndexOf( args[i] ) != -1 )
				{
					if ( i+1 < args.Length )
					{
						lookupTable.Add( args[i], args[i+1] );
						i++;
					}
					else
					{
						Console.WriteLine("Error, missing value for argument {0}.", args[i]);
						result = false;
					}
				}
				else
				{
					// An invalid argument was specified
					Console.WriteLine("Error, \"{0}\" is an invalid argument", args[i]);
					result = false;
				}
			}
		}
		return result;
	}

	/******************************************************************************
	 * GetHashTable - Returns the lookup table for command line arguments
	 */
	public Hashtable GetHashTable()
	{
		return lookupTable;
	}
	// Private member data
	private ArrayList singleArgs = new ArrayList();
	private ArrayList doubleArgs = new ArrayList();
	public readonly Hashtable lookupTable = new Hashtable();
}